###
# Application: pyFind
# File:        __init__.py
# Description: 
# Copyright (c) 2001-2002   Andy Balcombe <kinematics _at_ ntlworld.com>
###
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

"""pyFind package"""

#i18n stuff 
try:
    import gettext
    import commands
    
    domain = 'pyfind'
    
    data_dir = commands.getoutput('gnome-config --datadir')
    locale_dir = data_dir + '/locale'
    
    gettext.bindtextdomain(domain, locale_dir)
    gettext.textdomain(domain)
    
except IOError:
    print "IOError in __init__.py"
    pass
    
import __builtin__
__builtin__.__dict__['_'] = gettext.gettext



###
# Application: pyFind
# File:        LoadGlade.py
# Description: Superclass for glade based applications
# Copyright (c) 2001-2002   Andy Balcombe <kinematics _at_ ntlworld.com>
###
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#from gtk import * 
#from gnome.ui import *
import gtk
import gnome.ui
import gnome.util
import libglade

import os.path

class LoadGlade:
    """
    Superclass for glade based applications
    """
    def __init__( self, sFilename, sWindowName, sMode="production" ):
        #set some properties
        self.sFilename = sFilename
        self.sWindowName = sWindowName
        self.sMode = sMode
        #print "LoadGlade: self.sMode=",self.sMode

        #set glade file name
        if self.sMode == "development":
            #development version
            self.sGladeFile = os.path.join('glade', self.sFilename)
        else:
            #production version
            self.sGladeFile = gnome.util.datadir_file( os.path.join('pyFind/glade', self.sFilename)) 

        #load glade file
        self.xml = libglade.GladeXML(self.sGladeFile, self.sWindowName)

        #create signal dictionary
        dict = {}
        for key in dir(self.__class__):
            dict[key] = getattr(self, key)
        self.xml.signal_autoconnect (dict)


    def __getattr__( self, key ):
        #print "LoadGlade.__getattr__(", key, ")"
        w = self.xml.get_widget(key)
        if w is None:
            raise AttributeError("Widget " + key + " not found")
        self.__dict__[key] = w
        return w

    #set this
    __getitem__ = __getattr__



###
# Application: pyFind
# File:        file_srch_rep.py
# Description: Search and replace text within a given filename.
#              Modified by Andy Balcombe from original script 
#              by Jeff Bauer <jbauer@rubic.com>
#              from http://www.activestate.com/ASPN/Python/Cookbook
###
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import os
import shutil
import tempfile
import re
import string  # python 1.5.2

def search_and_replace( sSearchText, sReplaceText, sFilenameIn, sFilenameOut = None, bUseRE = 0, bIgnoreCase = 0 ):
    """
    Search and replace text in a given filename.
        Parameters:
            sSearchText  : string  : Text / regular expression to search for
            sReplaceText : string  : Replace search text with this
            sFilenameIn  : string  : Input filename
            sFilenameOut : string  : Output filename (if blank the input file will be overwritten)
            bUseRE       : boolean : True if sSearchText is a regular expression
            bIgnoreCase  : boolean : True to ignore case (regular expression searches only)
    Returns:
        boolean : True if found a match, False otherwise
    """
    #if no output filename specified, then overwrite input file using
    #a temporary filename in the process
    if sFilenameOut == None or sFilenameIn == sFilenameOut:
        bTemp = 1
        sFilenameOut = tempfile.mktemp()
        # print "temp file = ", sFilenameOut
    else:
        bTemp = 0
        
    #open files
    input = open(sFilenameIn)
    output = open(sFilenameOut, "w")

    #compile reg exp if needed
    if bUseRE:
        reExp = re.compile(sSearchText, reFlags)

    #search and replace
    for s in input.readlines():
        if bUseRE:
            #regular expression
            if bIgnoreCase:
                reFlags = re.IGNORECASE
            else:
                reFlags = 0
            #reExp = re.compile(sSearchText, reFlags)
            output.write( re.sub(reExp, sReplaceText, s) )
        else:
            #normal search / replace
            # output.write(s.replace(sSearchText, sReplaceText)) #python 2.0
            output.write(string.replace(s, sSearchText, sReplaceText))  # python 1.5.2 

    #close files
    input.close()
    output.close()

    #replace input file with temp output file if required
    if bTemp:
        shutil.copyfile( sFilenameOut, sFilenameIn )
        os.remove( sFilenameOut )



###
# Application: pyFind
# File:        config.py
# Description: Configuration class.
# Copyright (c) 2001-2002   Andy Balcombe <kinematics _at_ ntlworld.com>
###
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#standard python modules
import gnome.config

#version info for pyFind
app_name = "pyFind"
app_version = "0.10.2"
authors = [ "Andy Balcombe <kinematics@ntlworld.com>", 
            "Contributors:", 
            "     Michele Campeotto <micampe@f2s.com>",
            "     David Rasch <drasch@science.uva.nl>",
            "     John W. Shipman <john@nmt.edu>",
            "     Jeff Bauer <jbauer@rubic.com>",
            "Translators:",
            "     ru: Vladimir and Pavel (svo@asplinux.udm.net)",
            "Thanks to:",
            "     Jonathan Slavin (jslavin@cfa.harvard.edu)",
            "     Michele Campeotto <micampe@f2s.com>",
            "         and all the contributors to the excellent",
            "         Moleskine editor <http://www.moleskine.f2s.com>"
          ]
            

#use the gnome configuration api
class GnomeConfig: 
    #init config file
    def __init__( self, psApp ):
        #set application filename to look for in ~/.gnome directory
        self.config_filename = psApp
        
    # sync the gnome config file
    def sync( self ):
        gnome.config.sync_file( self.config_filename )

    #return string value of specified key
    def get_value( self, psSection, psKey, psDefault ):
        #get dictionary of all entries in specified section
        sValue = gnome.config.get_string( self.config_filename + "/" + psSection + "/" + psKey )
        if sValue != None:
            return sValue
        else:
            return psDefault
            
    #save string value of specified key
    def set_value( self, psSection, psKey, psValue ):
        gnome.config.set_string( self.config_filename + "/" + psSection + "/" + psKey, psValue )




###
# Application: pyFind
# File:        dlgPreferences.py
# Description: Preferences dialog
# Copyright (c) 2001-2002   Andy Balcombe <kinematics _at_ ntlworld.com>
###
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#project imports
from LoadGlade import *
import config

class dlgPreferences(LoadGlade):
    """
    pyfind preferences dialog class
    """
    def init_Preferences(self, app):
        #init prefs
        self.app = app
        self.dlgPreferences.set_parent(self.app.appMain)
        self.dlgPreferences.set_transient_for(self.app.appMain)

        #get config
        self.bLoading = 1
        self.confPrefs = config.GnomeConfig("pyFind")
        
        #set toolbar radio buttons
        iTbrMain = int(self.confPrefs.get_value( "config", "tbrMain", str(gtk.TOOLBAR_BOTH)))
        if iTbrMain == gtk.TOOLBAR_BOTH:
            self.optTbrMainBoth.set_active(gtk.TRUE)
        elif iTbrMain == gtk.TOOLBAR_TEXT:
            self.optTbrMainText.set_active(gtk.TRUE)
        elif iTbrMain == gtk.TOOLBAR_ICONS:
            self.optTbrMainIcons.set_active(gtk.TRUE)
        
        #iTbrFile = int(self.confPrefs.get_value( "config", "tbrFile", str(gtk.TOOLBAR_BOTH)))
        #if iTbrFile == gtk.TOOLBAR_BOTH:
        #    self.optTbrFileBoth.set_active(gtk.TRUE)
        #elif iTbrFile == gtk.TOOLBAR_TEXT:
        #    self.optTbrFileText.set_active(gtk.TRUE)
        #elif iTbrFile == gtk.TOOLBAR_ICONS:
        #    self.optTbrFileIcons.set_active(gtk.TRUE)

        #set columns tab
        sResultCols = self.confPrefs.get_value( "config", "ResultCols", "011100101")
        lCols = ["chkColType","","","chkColPermissions","chkColUser","chkColGroup","chkColSize","chkColATime","chkColMTime",]
        iCol = 0
        for sCol in sResultCols:
            if lCols[ iCol ] != "":
                #build up this codestring and execute it
                sExec = "self." + lCols[ iCol ] + ".set_active( int(sCol) )"
                exec sExec
            iCol = iCol +1
        
        #self.confPrefs.sync()

        #display prefs
        self.bLoading = 0
        self.dlgPreferences.show()
        
    def on_preferences_changed(self, *args):
        #tells gnome to call on_preferences_apply when OK or Apply pressed in dialog
        if not self.bLoading:
            self.dlgPreferences.changed() 

    def on_preferences_apply(self, ctlPropBox, page, *args):
            #quit if invalid page
        if (page != -1):
            return

        #init config values
        iTbrMain = gtk.TOOLBAR_ICONS
        iTbrFile = gtk.TOOLBAR_ICONS
        sResultsCols = ""

        # Check and save the display toolbar radio buttons (Main Toolbar)
        if self.optTbrMainBoth.get_active():
            iTbrMain = gtk.TOOLBAR_BOTH
        elif self.optTbrMainIcons.get_active():
            iTbrMain = gtk.TOOLBAR_ICONS
        elif self.optTbrMainText.get_active():
            iTbrMain = gtk.TOOLBAR_TEXT
        self.confPrefs.set_value( "config", "tbrMain", str(iTbrMain) )

        # Check and save the display toolbar radio buttons (File Toolbar)
        #if self.optTbrFileBoth.get_active():
        #    iTbrFile = gtk.TOOLBAR_BOTH
        #elif self.optTbrFileIcons.get_active():
        #    iTbrFile = gtk.TOOLBAR_ICONS
        #elif self.optTbrFileText.get_active():
        #    iTbrFile = gtk.TOOLBAR_TEXT
        #self.confPrefs.set_value( "config", "tbrFile", str(iTbrFile) )
    
        #check and save the results columns list
        sResultCols = str(self.chkColType.get_active()) + \
                      "11" + \
                      str(self.chkColPermissions.get_active()) + \
                      str(self.chkColUser.get_active()) + \
                      str(self.chkColGroup.get_active()) + \
                      str(self.chkColSize.get_active()) + \
                      str(self.chkColATime.get_active()) + \
                      str(self.chkColMTime.get_active())
        self.confPrefs.set_value( "config", "ResultCols", sResultCols )

        #sync config
        self.confPrefs.sync()
        
        #redisplay preferences
        self.app.set_config_preferences()
        
        


###
# Application: pyFind
# File:        dlgSaveAs.py
# Description: Preferences dialog
# Copyright (c) 2001-2002   Andy Balcombe <kinematics _at_ ntlworld.com>
###
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#project imports
from LoadGlade import *
import config

class dlgSaveAs(LoadGlade):
    """
    pyfind save as dialog class
    """
    def init_dlgSaveAs(self, app):
        #init prefs
        self.app = app
        self.dlgSaveAs.set_parent(self.app.appMain)
        self.dlgSaveAs.set_transient_for(self.app.appMain)

        #get config
        self.bLoading = 1

        #display save as dialog
        self.bLoading = 0
        self.dlgSaveAs.show()
        
    def on_cmdOK_clicked(self, *args):
        #save list of results
        #print "cmdOK clicked: ", self.dlgSaveAs.get_filename()
        #open file
        f = open( self.dlgSaveAs.get_filename(), "w" )
        #for each row
        for iRow in range(0,self.app.clstFiles.__getattr__("rows")):
            sFilename = self.app.clstFiles.get_text(iRow,2) + "/" + \
                        self.app.clstFiles.get_text(iRow,1)  + "\n"
            f.write(sFilename)
        #done
        f.close()
        self.dlgSaveAs.destroy()

    def on_cmdCancel_clicked(self, *args):
        print "cmdCancel clicked"
        self.dlgSaveAs.destroy()

        


###
# Application: pyFind
# File:        appMain.py
# Description: Main window callbacks
# Copyright (c) 2001-2002   Andy Balcombe <kinematics _at_ ntlworld.com>
###
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#standard python modules
import os

#project modules
from LoadGlade import *
import dlgPreferences
import dlgSaveAs
import config
import match_files

class appMain(LoadGlade):
    """
    The main pyfind gui class
    """
    def init_appMain(self, p_sPath=""):
        #init config db
        self.confPrefs = config.GnomeConfig("pyFind")
        
        #signal connect
        self.appMain.connect("destroy", gtk.mainquit)

        #set inital values
        self.apbMain.set_status("Started")
        self.cmdStop.set_sensitive(gtk.FALSE)
        self.set_file_buttons()
        
        #drag and drop stuff
        self.targets = [('STRING', 0, 0), ('text/plain', 0,1), ('text/uri-list', 0, 2) ]
        self.clstFiles.drag_dest_set(gtk.DEST_DEFAULT_ALL, self.targets, gtk.GDK.ACTION_COPY | gtk.GDK.ACTION_MOVE)
        self.clstFiles.drag_source_set(gtk.GDK.SHIFT_MASK | gtk.GDK.BUTTON1_MASK, self.targets, gtk.GDK.ACTION_COPY | gtk.GDK.ACTION_MOVE )
        #self.clstFiles.connect("drag_data_get",self.on_clstFiles_drag_data_get)

        #load histories
        self.entFileSpec.load_history()
        self.entContaining.load_history()
        self.fentSearchIn.gnome_entry().load_history()

        #see if --last has been used to run pyfind...
        if p_sPath == "":
            #...yes, start with previous settings
            sLastFileSpec = self.confPrefs.get_value( "History: entFileSpec", "0", "*" )
            sLastSearchItem = self.confPrefs.get_value( "History: entContaining", "0", "" )
            sLastSearchPath = self.confPrefs.get_value( "History: fentSearchIn", "0", "/" )
            self.txeFileSpec.set_text(sLastFileSpec) 
            self.txeContaining.set_text(sLastSearchItem) 
            self.txeSearchIn.set_text( sLastSearchPath )
        else:
            #...no, start with default settings and specified path
            self.txeFileSpec.set_text("*") 
            self.txeContaining.set_text("") 
            self.txeSearchIn.set_text( p_sPath )

        #init toolbars and result cols, etc
        self.set_config_preferences()
        
        #flags, etc
        self.bStopPressed = 0

    #process gtk / gnome events
    def process_gtk_events(self):
        while gtk.events_pending():
            gtk.mainiteration()

    def set_config_preferences(self):
        #init toolbars
        self.tbrMain.set_style( int(self.confPrefs.get_value( "config", "tbrMain", str(gtk.TOOLBAR_BOTH))))
        #self.tbrFile.set_style( int(self.confPrefs.get_value( "config", "tbrFile", str(gtk.TOOLBAR_BOTH))))

        #set columns
        sCols = self.confPrefs.get_value( "config", "ResultCols", "011100101" )
        iColNum = -1
        for sColVisible in sCols:
            iColNum = iColNum + 1
            self.clstFiles.set_column_visibility( iColNum, (sColVisible=="1") )

        #sync config file
        self.confPrefs.sync()

    def set_file_buttons(self):
        #get selected rows
        lSelected = self.clstFiles.selection
        if len(lSelected) > 0:
            self.cmdFOpen.set_sensitive(gtk.TRUE)
            self.cmdFDelete.set_sensitive(gtk.TRUE)
        else:
            self.cmdFOpen.set_sensitive(gtk.FALSE)
            self.cmdFDelete.set_sensitive(gtk.FALSE)

    ### Main Toolbar
    # Find
    def on_cmdFind_clicked(self, *args):
        #sanity checking
        if self.txeFileSpec.get_text() == "" or \
           self.fentSearchIn.get_full_path(1) == None:
            #stop here
            self.apbMain.set_status( "Enter a path and file spec" )
            return
            
        #search for files!
        self.entFileSpec.prepend_history(1,self.txeFileSpec.get_text())
        self.entContaining.prepend_history(1,self.txeContaining.get_text())
        self.fentSearchIn.gnome_entry().prepend_history(1,self.fentSearchIn.get_full_path(1))

        #set progress bar to 0 and clear list
        self.cmdStop.set_sensitive(gtk.TRUE)
        self.apbMain.set_progress(0.0)
        self.clstFiles.clear()
        self.process_gtk_events()

        #reset flags
        self.bStopPressed = 0

        #build parameter list
        if self.txeFileSize.get_text() == "At Least":
            sSizeType = ">"
        else:
            sSizeType = "<"

        #call search module
        lFiles = []
        lFiles, sFindCmd = match_files.find_matching_files(
            self,
            self.fentSearchIn.get_full_path(1),        #search path
            not(self.chkSubDirectories.get_active()),  #search sub-directories
            self.chkSearchMountedVols.get_active(),    #search mounted volumes
            self.txeFileSpec.get_text(),               #file spec 
            self.chkFileCase.get_active(),             #case sensitive filenames
            self.chkPreviousDays.get_active(),         #search previous 
            self.txeMDays.get_text(),                  #num previous type (weeks, days, etc)
            self.spnPreviousDays.get_value_as_int(),   #num previous val
            self.txeContaining.get_text() != "",       #search contents
            self.chkContCase.get_active(),             #case sensitive contents
            self.txeContaining.get_text(),             #search contents val
            self.chkFileSize.get_active(),             #search by size
            sSizeType,                                 #size type (< or >)
            self.spnFileSize.get_value_as_int() * 1024,#size in kb
            self.chkBetween.get_active(),              #search on mod date
            self.dteBetweenStart.get_date(),           #start date
            self.dteBetweenEnd.get_date() + 86400,     #end date +1 day (to make date inclusive) 
            self.chkReplace.get_active(),              #search and replace text within files
            self.txeRepSearch.get_text(),              #replace - search text
            self.txeRepReplace.get_text(),             #        - replace text
            self.chkRegExp.get_active(),               #        - use regular expressions
            self.chkRepCaseSensitive.get_active()      #        - case senstive search for regular expressions
        )

        #done search
        self.txeFindCmd.set_text(sFindCmd) #fill in find cmd line
        self.cmdStop.set_sensitive(gtk.FALSE)
        
    # Clear search parameters
    def on_cmdReset_clicked(self, *args):
        #clear all search options
        self.txeFileSpec.set_text("*")
        self.txeContaining.set_text("")
        self.txeSearchIn.set_text("/")

    #stop search
    def on_cmdStop_clicked(self, *args):
        #set flag (checked in match_files.py)
        self.bStopPressed = 1
        
    # save as
    def on_save_as_activate(self, *args):
        dlgSaveResults = dlgSaveAs.dlgSaveAs("pyfind.glade","dlgSaveAs", self.sMode)
        dlgSaveResults.init_dlgSaveAs(self)
        
    # display Preferences dialog
    def on_preferences_activate(self, *args):
        dlgPrefs = dlgPreferences.dlgPreferences("pyfind.glade","dlgPreferences", self.sMode)
        dlgPrefs.init_Preferences(self)
        
    # display About dialog 
    def on_about_activate(self, *args):
        #set glade file name
        if self.sMode == "development":
            #development version
            sPixmap = os.path.join("pixmaps", "pyFind3.png")
        else:
            #production version
            sPixmap = gnome.util.datadir_file( os.path.join("pyFind/pixmaps", "pyFind3.png")) 
        dlgAbout = gnome.ui.GnomeAbout( config.app_name, \
                       config.app_version, \
                       "GPL (c) 2001-2002 Andy Balcombe", \
                       config.authors, \
                       "pyFind - a simple Gnome file find utility", \
                       sPixmap )
        dlgAbout.show()

    # Open selected files
    def on_cmdFOpen_clicked(self, *args):
        #get the name of the Gnome editor
        confEditor = config.GnomeConfig("editor")
        #for each file in selection
        lSelected = self.clstFiles.selection
        sFilename = ""
        for iRow in lSelected:
            sFilename = sFilename + " " + \
                        self.clstFiles.get_text(iRow,2) + "/" + \
                        self.clstFiles.get_text(iRow,1) 
        #open the editor with a list of files
        sCmd = "gnome-edit %s &" % (sFilename)
        os.system( sCmd )

    # Delete selected files
    def on_cmdFDelete_clicked(self, *args):
        lSelected = self.clstFiles.selection
        self.lDelFiles=[] #empty list
        for iRow in lSelected:
            sFilename = self.clstFiles.get_text(iRow,2) + "/" + \
                        self.clstFiles.get_text(iRow,1) 
            self.lDelFiles.append([iRow,sFilename])
        if self.lDelFiles != []:
            sMsg = "Are you sure you want to delete the " + str(len(self.lDelFiles)) + " selected files?"
            ctlMsgBox = gnome.ui.GnomeQuestionDialog( sMsg, self.on_ctlMsgBox_question, self.appMain)

    # callback for delete files question box
    def on_ctlMsgBox_question(self, button, *args):
        #print "on_ctlMsgBox_question=",button
        if button == 0: #yes
            #print "self.lDelFiles=",self.lDelFiles
            i=0
            for iRow, sFile in self.lDelFiles:
                try:
                    os.remove( sFile )
                    self.clstFiles.remove( iRow-i )
                    i=i+1
                except:
                    sErrMsg = "Error - Cannot remove file:\n " + sFile
                    ctlMsgBox = GnomeErrorDialog(sErrMsg, self.appMain)
        #remove file list
        del self.lDelFiles

    # file list (column heading)
    def on_clstFiles_click_column(self, ctlCList, iCol, *args):
        #sort the results list
        self.clstFiles.set_sort_column(iCol)
        self.clstFiles.sort()

    def on_clstFiles_drag_drop(self, w, context, x, y, time):
        pass

    # called when a 'drop' occurs on a dnd target
    def on_clstFiles_drag_data_get(self,w,context,selection_data,info, time):
        sFiles="" #file list
        #for each selected file...
        lSelected = self.clstFiles.selection
        for iRow in lSelected:
            #basic filename
            sFilename = self.clstFiles.get_text(iRow,2) + "/" + \
                        self.clstFiles.get_text(iRow,1) 
            if selection_data.target == self.targets[2][0]:
                #text/uri-list
                sFilename = "file://" + sFilename + "\r\n"
            else:
                #STRING or text/plain
                sFilename = sFilename + "\n"
            #add to list
            sFiles = sFiles + sFilename
        selection_data.set(selection_data.target, 8, sFiles)

    def on_clstFiles_drag_begin(self, targets, actions):
        #start of a dnd thingy
        pass
        
    # file list (row selected)
    def on_clstFiles_select_row(self, ctlCList, iRow, iCol, iX, *args):
        self.set_file_buttons()

    # file list (row unselected)
    def on_clstFiles_unselect_row(self, ctlCList, iRow, iCol, iX, *args):
        self.set_file_buttons()

    # file filter tab
    # File Size
    def on_chkFileSize_toggled(self, *args):
        bEnabled = self.chkFileSize.get_active()
        self.cboFileSize.set_sensitive( bEnabled )
        self.spnFileSize.set_sensitive( bEnabled )
        self.lblKb.set_sensitive( bEnabled )

    # Previous
    def on_chkPreviousDays_toggled(self, *args):
        bEnabled = self.chkPreviousDays.get_active()
        self.spnPreviousDays.set_sensitive( bEnabled )
        self.cboMDays.set_sensitive( bEnabled )
        
    # Date Range
    def on_chkBetween_toggled(self, *args):
        bEnabled = self.chkBetween.get_active()
        self.dteBetweenStart.set_sensitive( bEnabled )
        self.dteBetweenEnd.set_sensitive( bEnabled )
        self.lblPrevious.set_sensitive( bEnabled )

    # Replace Tab
    # search and replace
    def on_chkReplace_toggled(self, *args):
        bEnabled = self.chkReplace.get_active()        
        self.chkRegExp.set_sensitive( bEnabled )
        self.lblRepSearch.set_sensitive( bEnabled )
        self.txeRepSearch.set_sensitive( bEnabled )
        self.lblRepReplace.set_sensitive( bEnabled )
        self.txeRepReplace.set_sensitive( bEnabled )

    # case sensitive reg exp matching
    def on_chkRegExp_toggled(self, *args):
        bEnabled = self.chkRegExp.get_active()        
        self.chkRepCaseSensitive.set_sensitive( bEnabled )

    #done, quit the application
    def on_appMain_destroy (self, *args):
        gtk.mainquit()




###
# Application: pyFind
# File:        match_files.py
# Description: File search function
# Copyright (c) 2001-2002   Andy Balcombe <kinematics _at_ ntlworld.com>
###
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#standard python modules
import os
import time
import stat
import pwd, grp

#project modules
import pathinfo
import file_srch_rep

"""
Function:    find_matching_files
Description: Returns a list of tuples of matched files.
Parameters:
    Main app window class       object (class of calling GUI)
    Search Path                 string
    Search Sub-directories      boolean
    Search Mounted Volumes      boolean
    File Spec                   string
    Case Sensitive              boolean
    Search by Previous Date     boolean
    Previous Date Type          string (min, day, week, month, year)
    Previous Date Number        integer
    Search File Contents        boolean
    Search Contents Case Sens   boolean
    Search File For             string
    Search by File Size         boolean
    File Size Type              string (<, >)
    File Size                   integer
    Search by Mod Date          boolean
    Min Date                    date
    Max Date                    date
    Search and Replace          boolean
    Replace - Search Text       string
    Replace - Replace Text      string
    Replace - Use Reg Exp       boolean
    Replace - Case Sensitive    boolean
    
Returns:
    List of files:
        Type
        Name
        Path
        Permission
        User
        Group
        Size
        Acc_time
        Mod_date
    Find command-line string
"""
def find_matching_files(
        pclsApp,
        psSearchPath,
        pbSearchSubDirs, 
        pbSearchMounted,
        psFileSpec, 
        pbCaseSensitive,
        pbSearchPrevDate,
        psPrevDateType,
        piPrevDateVal,
        pbSearchContents,
        pbContentsCaseSensitive,
        psSearchContentsFor,
        pbSearchSize,
        psSizeType,
        piSize,
        pbSearchModDate,
        pdatMinDate,
        pdatMaxDate,
        pbReplace,
        psRepSearch,
        psRepReplace,
        pbRepRE,
        pbRepCaseSensitive ):

    #init
    sFindCmd = ""
    sParam = ""
    sGrep = ""
    sGrepCmd = ""
    iVal = 0
    tFileInfo = ()
    lFindFiles = []     #files found in find command
    lFiles = []         #files matching all specified parameters
    lFileInfo = ["type","file","path","permission","user","group","size","acc_time","mod_date"]
    bFileMatch = 0      #true if file matches all criteria
    iNumFilesFound = 0  #number of matched files

    #setup GUI
    iProgress = 0.0
    #pclsApp.clstFiles.freeze() 
    pclsApp.apbMain.set_progress(iProgress)

    #search mounted volumes?
    if not pbSearchMounted:
         psSearchPath = '-x ' + psSearchPath

    #case sensitive file name search
    if pbCaseSensitive:
        sParam = ' -name "' #yes
    else:
        sParam = ' -iname "' #no
    sFindCmd = "find " + psSearchPath + sParam + psFileSpec + '"' 
    
    #search sub-directories
    if pbSearchSubDirs:
        sFindCmd = sFindCmd + " -maxdepth 1"
    
    #search previous days
    if pbSearchPrevDate:
        sParam = " -mtime -"
        iVal = piPrevDateVal
        if psPrevDateType == "Minutes":
            sParam = " -mmin -"
        elif psPrevDateType == "Weeks":
            iVal = iVal * 7
        elif psPrevDateType == "Months":
            iVal = iVal * 30
        elif psPrevDateType == "Years":
            iVal = iVal * 365
        # add mtime/mmin argument
        sFindCmd = "%s %s%i" % (sFindCmd, sParam, iVal)

    #output to command line
    sFindCmd = sFindCmd + " -print"
    
    #search contents?
    if pbSearchContents:       
        #case-sensitive contents
        if pbContentsCaseSensitive:
            sGrep = "grep " #yes
        else:
            sGrep = "grep -i " #no

    #search for files - execute find command
    for objFile in os.popen(sFindCmd).readlines():
        sFileName = objFile[:-1]
        bFileMatch = 0
        
        #check gtk / gnome events
        pclsApp.process_gtk_events()
        if pclsApp.bStopPressed:
            #stop pressed
            break

        #check contents of file?
        if pbSearchContents:
            sGrepCmd = sGrep + psSearchContentsFor + " " + sFileName
            #search contents of file...
            if len(os.popen(sGrepCmd).readlines()) > 0:
                #yep
                bFileMatch = 1
        else:
            #not checking contents, so add file to list anyway
            bFileMatch = 1

        #have we got an initial match
        if bFileMatch:
            #yes
            iNumFilesFound = iNumFilesFound + 1           
            #so, now do extra processing...
            bFileMatch = 1
            iFileSize    = os.path.getsize(sFileName)        #get file size
            datFileATime = os.path.getatime(sFileName)       #get acc time
            datFileMTime = os.path.getmtime(sFileName)       #get mod time
            tStat        = os.stat(sFileName)                #get stat info
            sPermissions = pathinfo.PathInfo(sFileName).permissions() #get permissions

            lFileInfo[0]=""                                  #type
            lFileInfo[1]=os.path.basename(sFileName)         #filename
            lFileInfo[2]=os.path.dirname(sFileName)          #directory
            lFileInfo[3]=sPermissions                        #permissions
            lFileInfo[4]=pwd.getpwuid(tStat[stat.ST_UID])[0] #user
            lFileInfo[5]=grp.getgrgid(tStat[stat.ST_GID])[0] #group
            lFileInfo[6]=str(iFileSize)                      #size
            lFileInfo[7]=time.ctime(datFileATime)            #acc time
            lFileInfo[8]=time.ctime(datFileMTime)            #mod time

            #check size
            if pbSearchSize:
                bFileMatch = 0
                if psSizeType == ">": # at least
                    bFileMatch = ( iFileSize > piSize )
                else:                  # at most
                    bFileMatch = ( iFileSize < piSize )
        
            #check file mod time
            if bFileMatch and pbSearchModDate:
                bFileMatch = 0
                if pdatMinDate != 0 and pdatMaxDate != 0:
                    if datFileMTime >= pdatMinDate and datFileMTime <= pdatMaxDate:
                        bFileMatch = 1
            
            #add the file to the results list
            if bFileMatch:
                #calc size
                fSize = float(lFileInfo[6]) / 1024
                if fSize < 10.0:
                    lFileInfo[6] = "%.1f" % fSize #convert size to (n.n)Kb
                else:
                    lFileInfo[6] = "%i" % fSize #convert size to (n)Kb
                    
                #copy file info to a tuple because its immutable!
                tFileInfo = tuple( lFileInfo ) 
                lFiles.append( tFileInfo )
                
                #search and replace?
                if pbReplace and psRepSearch != "":
                    #do it baby, yeah
                    #print "searching and replacing: ", sFileName
                    file_srch_rep.search_and_replace( 
                        psRepSearch, 
                        psRepReplace, 
                        sFileName, 
                        None, 
                        pbRepRE,
                        pbRepCaseSensitive )
                
                #update GUI
                iProgress = iProgress + 0.1
                if iProgress > 1.0:
                    iProgress = 0.0
                pclsApp.apbMain.set_progress( iProgress )
                pclsApp.clstFiles.append( tFileInfo )
                sFound = "%s %i %s" % ( "Found ", len(lFiles), " files.")
                pclsApp.apbMain.set_status( sFound )

    #done - GUI
    #pclsApp.clstFiles.thaw()
    if pclsApp.bStopPressed:
        sMsg = "Interrupted.  Found "
    else:
        sMsg = "Done.  Found "
    sFound = "%s %i %s" % ( sMsg, len(lFiles), " files.")
    pclsApp.apbMain.set_progress(0.0)
    pclsApp.apbMain.set_status( sFound )

    # done!
    #print "find_matching_files found", len(lFiles), "files"
    return lFiles, sFindCmd
    



###
# Application: pyFind
# File:        pathinfo.py
# Description: Object for manipulating information about pathnames
# Written by:  John W. Shipman (john@nmt.edu)
###
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


"""pathinfo.py:  Object for manipulating information about pathnames
    and their related i-nodes.

    Written by John W. Shipman (john@nmt.edu), New Mexico Tech
    Computer Center, Socorro, NM 87801, in March 1997.

    Freely available under the terms of the GNU General Public
    License.

    This class defines methods for the ten standard components
    of the os.stat() value, giving information about file status.

    Exported methods:
================================================================
    PathInfo(pathName)
        [ if pathname is a valid, existing path name ->
            return a PathInfo representing that path and the
                associated i-node
          else ->
            raise an exception
        ]
    .__str__()
        [ return a string of the form "m  t  p" where:
            m  :=  "drwxrwxrwx" part of `ls -l'
            t  :=  time as "yyyy-mm-dd hh:mm:ss"
            p  :=  pathname used to create self
        ]
    .size()
        [ return self's size as an integer
        ]
    .ownerCanRead()
        [ if owner can read self -> return a nonzero integer
          else -> return 0
        ]
    .ownerCanWrite()
        [ if owner can write self -> return a nonzero integer
          else -> return 0
        ]
    .ownerCanExec()
        [ if owner can execute self -> return a nonzero integer
          else -> return 0
        ]
    .groupCanRead()
        [ if group can read self -> return a nonzero integer
          else -> return 0
        ]
    .groupCanWrite()
        [ if group can write self -> return a nonzero integer
          else -> return 0
        ]
    .groupCanExec()
        [ if group can execute self -> return a nonzero integer
          else -> return 0
        ]
    .worldCanRead()
        [ if world can read self -> return a nonzero integer
          else -> return 0
        ]
    .worldCanWrite()
        [ if world can write self -> return a nonzero integer
          else -> return 0
        ]
    .worldCanExec()
        [ if world can execute self -> return a nonzero integer
          else -> return 0
        ]
    .isDir()
        [ if self is a directory -> return a nonzero integer
          else -> return 0
        ]
    .isLink()
        [ if self is a symbolic link -> return a nonzero integer
          else -> return 0
        ]
    .isFile()
        [ if self is an ordinary disk file -> return a nonzero integer
          else -> return 0
        ]
"""

import os               # Operating system functions
import time             # Functions for handling time
import stat             # Functions for interpreting file status tuples

class PathInfo:
    """Object to represent a pathname and the information from the
        associated inode.
    """

#----------------------------------------------------------------
# State
#----------------------------------------------------------------
# .path == The path passed to __init__
#--
# .stat == The file status tuple
#--
# .mode == The mode bits from the file status tuple
#--

    def __init__ ( self, pathName ):
        self.path  =  pathName
        self.stat  =  os.lstat(pathName)
        self.mode  =  self.stat[stat.ST_MODE]

    def __str__ ( self ):
        baseName = (os.path.split(self.path))[1]
        return ( "%s  %s  %8d %s" %
                 ( self._modeFlags(), self._modTime(), self.size(),
                   baseName ) )

    def _modeFlags ( self ):
        return self._dirFlag() + self._permFlags()

    def _dirFlag ( self ): 
        """ [ if self is a directory -> return "d"
              else if self is a symbolic link -> return "l"
              else if self is a regular file -> return "-"
              else -> return "?"
            ]
        """
        if stat.S_ISLNK(self.mode):
            return "l"
        elif stat.S_ISDIR(self.mode):
            return "d"
        elif stat.S_ISREG(self.mode):
            return "-"
        else:
            return "?"

    def _permFlags ( self ):
        """Return the `rwxrwxrwx' part of ls output
        """
        return ( self._rwx ( self.mode & stat.S_IRUSR,
                             self.mode & stat.S_IWUSR,
                             self.mode & stat.S_IXUSR ) +
                 self._rwx ( self.mode & stat.S_IRGRP,
                             self.mode & stat.S_IWGRP,
                             self.mode & stat.S_IXGRP ) +
                 self._rwx ( self.mode & stat.S_IROTH,
                             self.mode & stat.S_IWOTH,
                             self.mode & stat.S_IXOTH ) )

    def _rwx ( self, r, w, x ):
        """ [ return a string of the form "abc" where:
                a  :=  "r" if r is nonzero, else "-"
                b  :=  "w" if w is nonzero, else "-"
                c  :=  "x" if x is nonzero, else "-"
            ]
        """
        return ( self._dasher ( r, "r" ) +
                 self._dasher ( w, "w" ) +
                 self._dasher ( x, "x" ) )

    def _dasher ( self, b, c ):
        """ [ if b is true -> return c
              else -> return "-"
            ]
        """
        if b:
            return c
        else:
            return "-"

    def _modTime ( self ):
        """ [ return self's modification time as "yyyy-dd-mm hh:mm:ss"
            ]
        """
        return time.strftime ( "%Y-%m-%d %H:%M:%S",
                               time.localtime ( self.stat[stat.ST_MTIME] ) )

    def size ( self ):
        return self.stat[stat.ST_SIZE]

    def permissions( self ):
        return self._dirFlag() + self._permFlags()

    def ownerCanRead ( self ):
        return self.mode & stat.S_IRUSR

    def ownerCanWrite ( self ):
        return self.mode & stat.S_IWUSR

    def ownerCanExec ( self ):
        return self.mode & stat.S_IXUSR

    def groupCanRead ( self ):
        return self.mode & stat.S_IRGRP

    def groupCanWrite ( self ):
        return self.mode & stat.S_IWGRP

    def groupCanExec ( self ):
        return self.mode & stat.S_IXGRP

    def worldCanRead ( self ):
        return self.mode & stat.S_IROTH

    def worldCanWrite ( self ):
        return self.mode & stat.S_IWOTH

    def worldCanExec ( self ):
        return self.mode & stat.S_IXOTH

    def isDir ( self ):
        return stat.S_ISDIR ( self.mode )

    def isLink ( self ):
        return stat.S_ISLNK ( self.mode )

    def isFile ( self ):
        return stat.S_ISREG ( self.mode )
